// Copyright  2008 for Adam Tibi , www.AdamTibi.NET . All Rights Reserved
using System;
using System.ComponentModel;
using System.Security.Permissions;
using System.Web;
using System.Web.UI.WebControls;
using System.Web.UI;
using System.Web.UI.HtmlControls;

namespace AT.Web.UI.Validators {

    [AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    [AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    [ToolboxData(@"<{0}:CheckValidator runat=server></{0}:CheckValidator>")]
    public class CheckValidator : BaseValidator {

        protected override bool EvaluateIsValid() {
            bool state = GetControlState();

            // state       warnif       return
            // -------------------------------
            // true         true        false
            // true         false       true
            // false        true        true
            // false        false       false
            // The result match an XOR ( ^ in C#) logical operator

            return state ^ WarnIf;
        }

        protected override bool ControlPropertiesValid() {

            // We need to override the ControlPropertiesValid to supress the error message that the 
            // BaseValidator will issue which states that you can not validate this control.

            if (ControlToValidate.Trim().Length == 0) {
                throw new HttpException(
                    string.Format("The ControlToValidate property of {0} cannot be blank.", ID));
            }

            Control control = FindControl(ControlToValidate);

            if (control == null) {
                throw new HttpException(
                    string.Format("Could not locate control with ID {0}", ID));
            }

            if (!(control is CheckBox) && !(control is RadioButton) && !(control is HtmlInputCheckBox) && !(control is HtmlInputRadioButton)) {
                throw new HttpException(
                    @"ControlToValidate can only be 
                        System.Web.UI.WebControls.CheckBox, 
                        System.Web.UI.WebControls.RadioButton, 
                        System.Web.UI.HtmlControls.HtmlInputCheckBox, 
                        System.Web.UI.HtmlControls.HtmlInputRadioButton");
            }

            return true;
        }

        protected override void AddAttributesToRender(HtmlTextWriter writer) {

            base.AddAttributesToRender(writer);

            if (RenderUplevel) {
                string clientID = ClientID;

                Page.ClientScript.RegisterExpandoAttribute(clientID, "evaluationfunction",
                                                           "CheckValidatorEvaluateIsValid");

                Page.ClientScript.RegisterExpandoAttribute(clientID, "warnif", WarnIf ? "1" : "0");

            }
        }

        private bool GetControlState() {
            
            Control control = FindControl(ControlToValidate);
            if (control is CheckBox) {
                return ((CheckBox)control).Checked;
            }
            
            if (control is RadioButton) {
                return ((RadioButton)control).Checked;
            }
            if (control is HtmlInputCheckBox) {
                return ((HtmlInputCheckBox)control).Checked;
            }
            if (control is HtmlInputRadioButton) {
                return ((HtmlInputRadioButton)control).Checked;
            }

            throw new HttpException("Could not determine the state of the control");
        }

        /// <summary>
        /// Triggers the validator based on checked (true) or unchecked (false) state
        /// </summary>
        [Browsable(true)]
        [Category("Behavior")]
        [Themeable(false)]
        [DefaultValue(false)]
        [Description("Triggers the validator based on checked (true) or unchecked (false) state")]
        public bool WarnIf {
            get {
                return (bool)(ViewState["WarnIf"] ?? false);
            }
            set {
                ViewState["WarnIf"] = value;
            }
        }

        protected override void OnPreRender(EventArgs e) {
            base.OnPreRender(e);
            if (base.RenderUplevel) {
                this.Page.ClientScript.RegisterClientScriptResource(typeof(CheckValidator), "AT.Web.UI.Validators.JavaScript.js");
            }
        }

    }
}
